#!/usr/bin/env python
#-*- coding:utf-8 -*-
#===============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#===============================================================================

""" Generator for API commands according to ONTAPI version. """

import os

from schtob.pyontapi import errors
from schtob.pyontapi.generator import system, types, GENERIC_TYPEDEFS

def generate(filer):
    """ generate API commands for ``filer``'s version. """
    
    api = system.System(filer)
    
    result = filer.get_ontapi_version()
    ontapi_version = '%s_%s' % (
        result['major-version'],
        result['minor-version']
    )
    
    typedefs = types.gen_typedefs(api)
    packages = get_api_command_packages(api, typedefs)
    
    path = os.path.abspath(os.path.join(
        os.path.dirname(__file__), os.pardir, 'api', filer.packagefilername,
        'ontapi_%s' % ontapi_version
    ))
    
    _path = os.path.abspath(os.path.join(
        os.path.dirname(__file__), os.pardir, 'api', filer.packagefilername
    ))
    if not os.path.exists(_path):
        os.makedirs(_path)
    if not os.path.exists(path):
        os.makedirs(path)
    
    handle = file(os.path.join(_path, '__init__.py'), 'w')
    handle.write('''#!/usr/bin/env python
#-*- coding:utf-8 -*-
#===============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#===============================================================================

"""
ONTAPI Version api modules for filer %(filer)s
"""

''' % {'filer': filer.filer})
    
    handle.close()
    
    handle = file(os.path.join(path, '__init__.py'), 'w')
    handle.write('''#!/usr/bin/env python
#-*- coding:utf-8 -*-
#===============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#===============================================================================

"""
ONTAPI Version %(version)s api modules for filer %(filer)s
"""

''' % {'version': ontapi_version, 'filer': filer.filer})
    for package in packages:
        handle.write('from schtob.pyontapi.api.%s.ontapi_%s.%s import %s\n' % (
            filer.packagefilername, ontapi_version, 
            package, package.capitalize()
        ))
    handle.write('\n\nAPI_CLASSES = (\n')
    for package in packages:
        handle.write('    %s,\n' % package.capitalize())
    handle.write(')\n\n')
    handle.close()
    
    for package, commands in packages.items():
        handle = file(os.path.join(path, '%s.py' % package), 'w')
        handle.write('''#!/usr/bin/env python
#-*- coding:utf-8 -*-
#===============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#===============================================================================

"""
%(package)s api module.
"""

from schtob.pyontapi.api.base import BaseAPI, NaArgument, NaField, NaTypeElement
from schtob.pyontapi.api.base import NaNamedType

class %(Package)s(BaseAPI):
    
    """ %(package)s api class """

    __api_name__ = '%(package)s'
    
''' % {
        'version': ontapi_version,
        'package': package,
        'Package': package.capitalize(),
})
        for cmd in commands:
            handle.write(cmd.get_py_method())
        handle.close()
    
def get_api_command_packages(api, typedefs):
    """
    Get all api commands for ``filer``. Returns a dict containing all 
    packages.
    """
    
    try:
        result = api.api_list()
    except errors.APIFailure, exc:
        raise errors.OntapiVersionError(exc.errno, 
                'Can not run system-api-list. Error was:\n%s' % exc.reason
        )
    
    try:
        result = api.api_get_elements([
            child['name'] for child in result['apis']
        ])
    except errors.APIFailure, exc:
        raise errors.OntapiVersionError(exc.errno, 
                    'Can not get api elements. Error was:\n%s' % exc.reason
        )
    
    
    packages = {}
    
    for child in result['api-entries']:
        name = child['name']
        elements = []
        if 'api-elements' in child:
            for element_info in child['api-elements']:
                info = {}
                for key, value in element_info.items():
                    info[key] = value
                var_type = info[u'type'].replace('[]', '')
                if var_type in GENERIC_TYPEDEFS:
                    var_type = GENERIC_TYPEDEFS[var_type]
                else:
                    var_type = typedefs[var_type].duplicate()
                typedef = types.TypeDef(info, var_type)
                if isinstance(var_type, types.NamedType):
                    var_type.set_output_value_for_childs(
                        typedef.is_optional
                    )
                elements.append(typedef)
        api_command = types.APICommand(name, elements)
        if not api_command.get_package() in packages:
            packages[api_command.get_package()] = []
        packages[api_command.get_package()].append(api_command)
    return packages
